<?php

declare(strict_types=1);

namespace Drupal\cheeseburger_menu;

use Drupal\Component\Utility\Html;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Template\Attribute;

/**
 * Cheeseburger menu class.
 */
class CheeseburgerMenu implements \IteratorAggregate {

  /**
   * Menu items.
   *
   * @var \Drupal\cheeseburger_menu\CheeseburgerMenuItem[]
   */
  protected array $menuItems = [];

  /**
   * Navigation meni items.
   *
   * @var \Drupal\cheeseburger_menu\CheeseburgerMenuItem[]
   */
  protected array $navigationMenuItems = [];

  /**
   * Is title hidden.
   *
   * @var bool
   */
  public bool $hiddenTitle = FALSE;

  /**
   * Is navigation title hidden.
   *
   * @var bool
   */
  public bool $navigationTitleHidden = FALSE;

  /**
   * Menu weight.
   *
   * @var int
   */
  public int $weight;

  /**
   * Entity type id from which the menu is originally created.
   *
   * @var string
   */
  protected string $originalEntityTypeId;

  /**
   * Entity id from which the menu is originally created.
   *
   * @var string|int
   */
  protected string|int $originalEntityId;

  /**
   * Unique menu id.
   *
   * @var string
   */
  public string $id;

  /**
   * Navigation item attribute.
   *
   * @var \Drupal\Core\Template\Attribute
   */
  public Attribute $navigationItemAttribute;

  /**
   * Menu title attribute.
   *
   * @var \Drupal\Core\Template\Attribute
   */
  public Attribute $titleAttribute;

  /**
   * Menu wrapper attributes.
   *
   * @var \Drupal\Core\Template\Attribute
   */
  public Attribute $wrapperAttribute;

  /**
   * Menu icon.
   *
   * @var string
   */
  public string $icon;

  /**
   * Cache contexts.
   *
   * @var string[]
   */
  protected array $cacheContexts = [];

  /**
   * Cheeseburger menu constructor.
   */
  public function __construct(
    public TranslatableMarkup|string $title,
  ) {
    $this->navigationItemAttribute = new Attribute(['class' => ['cheeseburger-menu__side-menu-item']]);
    $this->titleAttribute = new Attribute(['class' => ['cheeseburger-menu__title']]);
    $this->wrapperAttribute = new Attribute();
  }

  /**
   * Creates menu from settings.
   *
   * @param array $settings
   *   Settings to create menu from.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Entity as source, menu and taxonomy vocabulary are supported.
   *
   * @return static
   *   A new cheeseburger menu object.
   */
  public static function createFromSettings(array $settings, EntityInterface $entity): static {
    $instance = new static($settings['settings']['override_title'] ? $settings['settings']['title_override'] : $entity->label());

    if (!$settings['settings']['show_title_in_navigation']) {
      $instance->hideNavigationTitle();
    }
    if (!$settings['settings']['show_title_above_menu']) {
      $instance->hideTitle();
    }

    if ($settings['settings']['collapsible_title'] ?? FALSE) {
      $instance->wrapperAttribute->addClass('cheeseburger-menu__title--collapsible');
      if ($settings['settings']['title_default_expanded'] ?? TRUE) {
        $instance->wrapperAttribute->addClass('cheeseburger-menu__title--expanded');
      }
    }

    $instance->setIcon($settings['settings']['icon'])
      ->setOriginalEntityId($entity->id())
      ->setOriginalEntityTypeId($entity->getEntityTypeId())
      ->setWeight($settings['weight']);
    $instance->id = Html::cleanCssIdentifier($entity->getEntityTypeId() . '-' . $entity->id());
    return $instance;
  }

  /**
   * Returns unique menu id.
   *
   * @return string
   *   Unique menu id.
   */
  public function getId(): string {
    return $this->id;
  }

  /**
   * Cheeseburger menu as attribute.
   *
   * @return \Drupal\Core\Template\Attribute
   *   Menu id attribute.
   */
  public function getIdAsAttribute(): Attribute {
    return new Attribute([
      'data-cheeseburger-id' => $this->getId(),
    ]);
  }

  /**
   * Append menu item to menu.
   *
   * @param \Drupal\cheeseburger_menu\CheeseburgerMenuItem $menu_item
   *   Menu item.
   *
   * @return static
   *   The object itself for chaining
   */
  public function addMenuItem(CheeseburgerMenuItem $menu_item): static {
    $this->menuItems[] = $menu_item;
    $this->mergeCacheContext($menu_item->getCacheContexts());
    return $this;
  }

  /**
   * Sets menu items.
   *
   * @param \Drupal\cheeseburger_menu\CheeseburgerMenuItem[] $menu_items
   *   Menu items.
   *
   * @return static
   *   The object itself for chaining
   */
  public function setMenuItems(array $menu_items): static {
    $this->menuItems = $menu_items;
    return $this;
  }

  /**
   * Returns menu items.
   *
   * @return \Drupal\cheeseburger_menu\CheeseburgerMenuItem[]
   *   Menu items.
   */
  public function getMenuItems(): array {
    return $this->menuItems;
  }

  /**
   * Menu items.
   */
  public function getIterator(): \Traversable {
    return new \ArrayIterator($this->menuItems);
  }

  /**
   * Append menu item to navigation.
   *
   * @param \Drupal\cheeseburger_menu\CheeseburgerMenuItem $menu_item
   *   Menu item.
   *
   * @return static
   *   The object itself for chaining.
   */
  public function addNavigationMenuItem(CheeseburgerMenuItem $menu_item): static {
    $this->navigationMenuItems[] = $menu_item;
    return $this;
  }

  /**
   * Sets navigation menu items.
   *
   * @param \Drupal\cheeseburger_menu\CheeseburgerMenuItem[] $menu_items
   *   Menu items.
   *
   * @return static
   *   The object itself for chaining
   */
  public function setNavigationMenuItems(array $menu_items): static {
    $this->navigationMenuItems = $menu_items;
    return $this;
  }

  /**
   * Returns menu items.
   *
   * @return \Drupal\cheeseburger_menu\CheeseburgerMenuItem[]
   *   Menu items.
   */
  public function getNavigationMenuItems(): array {
    return $this->navigationMenuItems;
  }

  /**
   * Returns menu title.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup|string
   *   Menu title.
   */
  public function getTitle(): string|TranslatableMarkup {
    return $this->title;
  }

  /**
   * Sets menu title.
   *
   * @param string|\Drupal\Core\StringTranslation\TranslatableMarkup $title
   *   Menu title.
   */
  public function setTitle(string|TranslatableMarkup $title): void {
    $this->title = $title;
  }

  /**
   * Returns whether menu has title or not.
   *
   * @return bool
   *   TRUE if has title, FALSE otherwise.
   */
  public function isTitleHidden(): bool {
    return $this->hiddenTitle;
  }

  /**
   * Returns whether menu has title or not.
   *
   * @return bool
   *   TRUE if has title, FALSE otherwise.
   */
  public function isNavigationTitleHidden(): bool {
    return $this->navigationTitleHidden;
  }

  /**
   * Hides title.
   *
   * @return static
   *   The object itself for chaining.
   */
  public function hideTitle(): static {
    $this->hiddenTitle = TRUE;
    return $this;
  }

  /**
   * Hides title.
   *
   * @return static
   *   The object itself for chaining.
   */
  public function hideNavigationTitle(): static {
    $this->navigationTitleHidden = TRUE;
    return $this;
  }

  /**
   * Menu weight.
   *
   * @return int
   *   Menu weight.
   */
  public function getWeight(): int {
    return $this->weight;
  }

  /**
   * Set menu weight.
   *
   * @param int $weight
   *   Set menu weight.
   */
  public function setWeight(int $weight): static {
    $this->weight = $weight;
    return $this;
  }

  /**
   * Set menu icon.
   *
   * @param string $icon_content
   *   Icon content.
   *
   * @todo add jpg and png support.
   */
  public function setIcon(string $icon_content): static {
    $this->icon = $icon_content;
    return $this;
  }

  /**
   * Returns menu icon content.
   *
   * @return string
   *   Icon content.
   */
  public function getIcon(): string {
    return $this->icon;
  }

  /**
   * Menu has icon.
   *
   * @return bool
   *   Menu has icon.
   */
  public function hasIcon(): bool {
    return !empty($this->icon);
  }

  /**
   * Sets original entity id.
   *
   * @param int|string $original_entity_id
   *   Original entity ID.
   *
   * @return static
   *   Returns itself for chaining.
   */
  public function setOriginalEntityId(int|string $original_entity_id): static {
    $this->originalEntityId = $original_entity_id;
    return $this;
  }

  /**
   * Returns original entity ID.
   *
   * @return int|string
   *   Original entity ID.
   */
  public function getOriginalEntityId(): int|string {
    return $this->originalEntityId;
  }

  /**
   * Sets original entity type id.
   *
   * @param string $original_entity_type_id
   *   Original entity type id.
   */
  public function setOriginalEntityTypeId(string $original_entity_type_id): static {
    $this->originalEntityTypeId = $original_entity_type_id;
    return $this;
  }

  /**
   * Returns original entity type id.
   *
   * @return string
   *   Original entity type id.
   */
  public function getOriginalEntityTypeId(): string {
    return $this->originalEntityTypeId;
  }

  /**
   * Merges cache contexts.
   *
   * @param string[] $cache_contexts
   *   Cache contexts to merge.
   *
   * @return static
   *   Returns itself for chaining.
   */
  public function mergeCacheContext(array $cache_contexts): static {
    $this->cacheContexts = Cache::mergeContexts($cache_contexts, $this->cacheContexts);
    return $this;
  }

  /**
   * Returns ache contexts.
   *
   * @return string[]
   *   Cache contexts.
   */
  public function getCacheContexts(): array {
    return $this->cacheContexts;
  }

  /**
   * Sets cache contexts.
   *
   * @param array $cache_contexts
   *   Sets cache contexts.
   *
   * @return static
   *   Returns itself for chaining.
   */
  public function setCacheContext(array $cache_contexts): static {
    $this->cacheContexts = $cache_contexts;
    return $this;
  }

}
